==> unicore-uvos-server-1.5.1-0.src.tar.gz: src/main/java/pl/edu/icm/unicore/uvos/engine/VOApplicationModule.java <==
/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Mar 18, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.engine;

import java.util.List;

import javax.security.auth.x500.X500Principal;

import org.apache.ibatis.session.SqlSession;
import org.apache.log4j.Logger;
import org.bouncycastle.jce.PKCS10CertificationRequest;

import pl.edu.icm.unicore.uvos.api.ApplicationActions;
import pl.edu.icm.unicore.uvos.api.ApplicationStatus;
import pl.edu.icm.unicore.uvos.api.Group;
import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.api.IdentityType;
import pl.edu.icm.unicore.uvos.api.Permissions;
import pl.edu.icm.unicore.uvos.api.VOApplication;
import pl.edu.icm.unicore.uvos.api.VOApplicationForm;
import pl.edu.icm.unicore.uvos.api.exceptions.GroupNotKnownException;
import pl.edu.icm.unicore.uvos.api.exceptions.IdentityNotKnownException;
import pl.edu.icm.unicore.uvos.api.exceptions.InternalException;
import pl.edu.icm.unicore.uvos.api.exceptions.InvalidValueException;
import pl.edu.icm.unicore.uvos.api.exceptions.SecurityException;
import pl.edu.icm.unicore.uvos.api.exceptions.UVOSException;
import pl.edu.icm.unicore.uvos.api.exceptions.UnsupportedTypeException;
import pl.edu.icm.unicore.uvos.api.exceptions.VOApplicationException;
import pl.edu.icm.unicore.uvos.db.DB;
import pl.edu.icm.unicore.uvos.db.DBApps;
import pl.edu.icm.unicore.uvos.db.DBEntities;
import pl.edu.icm.unicore.uvos.db.DBGroupResolver;
import pl.edu.icm.unicore.uvos.db.DBGroups;
import pl.edu.icm.unicore.uvos.engine.api.VOApplicationInterface;
import pl.edu.icm.unicore.uvos.idhelpers.IdentityHelperFactory;
import pl.edu.icm.unicore.uvos.idhelpers.IdentityTypeHelper;
import pl.edu.icm.unicore.uvos.idhelpers.X509IdentityHelper;
import pl.edu.icm.unicore.uvos.mail.MailSender;
import pl.edu.icm.unicore.uvos.model.GroupBean;
import pl.edu.icm.unicore.uvos.model.VOApplicationBean;

/**
 * Class which handles processing of VO applications (or join requests). 
 * @author K. Benedyczak
 */
public class VOApplicationModule implements VOApplicationInterface
{
	private DB db;
	private DBGroupResolver groupResolver;
	private DBEntities entities;
	private DBApps myDb;
	private InternalAuthorization authZ;
	private MailSender mail;
	private DBGroups groups;
	private static final Logger log = Logger.getLogger(VOApplicationModule.class);
	
	public VOApplicationModule(DB db, DBGroupResolver groupResolver, DBEntities entities, DBApps myDb,
			InternalAuthorization authZ, MailSender mail, DBGroups groups)
	{
		this.db = db;
		this.myDb = myDb;
		this.authZ = authZ;
		this.mail = mail;
		this.groups = groups;
		this.groupResolver = groupResolver;
		this.entities = entities;
	}

	//
	// public API implementation ******************************************************
	//
	public void updateApplicationForm(VOApplicationForm applicationDef, boolean update) throws SecurityException, InternalException, 
			VOApplicationException
	{
		int baseGroup;
		SqlSession session = db.getSqlSession();
		try
		{
			try
			{
				GroupBean gb;
				if (!update)
					gb = groupResolver.resolveGroup(applicationDef.getBaseGroup(), null, session); 
				else
				{
					gb = groupResolver.resolveGroup(applicationDef.getBaseGroup(), null, session);
					baseGroup = gb.getId();
					gb = myDb.getAppFormGroup(applicationDef.getId(), session);
					if (baseGroup != gb.getId())
						throw new VOApplicationException("Application form update " +
						"can not change the group of the original application.");
				}
				authZ.authorize(Permissions.WRITE_P, gb, false, session);
				baseGroup = gb.getId();
			} catch (GroupNotKnownException e)
			{
				throw new VOApplicationException("Requested base group is unknown.");
			}

			if (update)
				myDb.updateAppForm(applicationDef, baseGroup, session);
			else
				myDb.addAppForm(applicationDef, session);
			session.commit();
		} finally
		{
			db.releaseSqlSession(session);
		}
	}

	public void removeApplicationForm(int id) 
		throws SecurityException, InternalException, VOApplicationException
	{
		SqlSession session = db.getSqlSession();
		try
		{
			try
			{
				GroupBean gb = myDb.getAppFormGroup(id, session);
				authZ.authorize(Permissions.WRITE_P, gb, false, session);
			} catch (GroupNotKnownException e)
			{
				throw new InternalException("Base group in application form is unknown.");
			}
			myDb.removeAppForm(id, session);
			session.commit();
		} finally
		{
			db.releaseSqlSession(session);
		}
	}

	public List<VOApplicationForm> getApplicationForms() 
		throws SecurityException, InternalException
	{
		SqlSession session = db.getSqlSession();
		try
		{
			return myDb.listAppForms(session);
		} finally
		{
			db.releaseSqlSession(session);
		}
	}

	public void submitApplication(VOApplication application) 
		throws SecurityException, InternalException, VOApplicationException
	{
		SqlSession session = db.getSqlSession();
		try
		{
			//this also checks if app form exists and is valid
			VOApplicationForm form = myDb.resolveAppForm(application.getFormId(), session);
			
			Identity id = application.getIdentity();
			IdentityTypeHelper helper = null;
			try
			{
				if (application.isCSR())
					X509IdentityHelper.parseCSR(id.getValue());
				else
					helper = IdentityHelperFactory.getHelper(id);
			} catch (Exception e)
			{
				throw new VOApplicationException("The identity value in " +
					"application is invalid for the specified type:" + 
					e);
			}
			Identity equivalent = application.getEquivalentIdentity();
			if (equivalent != null)
			{
				try
				{
					if (!authZ.isSelf(equivalent, session))
						throw new SecurityException("VO Applications for entry equivalent " +
							"to the existing one are available only for users " +
							"authenticated as this existing one.");
				} catch (IdentityNotKnownException e)
				{
					throw new VOApplicationException("Requested equivalent identity is not known.");
				} catch (InvalidValueException e)
				{
					throw new VOApplicationException("Requested equivalent identity is invalid.");
				}
			}
			
			myDb.addApplication(application, helper, session);
			session.commit();
			
			mail.sendNewApplicationNotification(application, form);
		} catch (UnsupportedTypeException e)
		{
			throw new VOApplicationException("Identity type requested in " +
					"application is not known");
		} finally
		{
			db.releaseSqlSession(session);
		}
	}

	public void processApplication(int id, ApplicationActions action, String notes, 
			boolean sendConfirmation, 
			VOApplication properApplication) 
		throws SecurityException, InternalException, VOApplicationException
	{
		SqlSession session = db.getSqlSession();

		try
		{
			VOApplication app = myDb.resolveApplication(id, session);
			GroupBean gb = myDb.getAppFormGroup(app.getFormId(), session);
			VOApplicationForm form = myDb.resolveAppForm(app.getFormId(), session);
			Group base;
			try
			{
				base = groupResolver.resolveGroupBean(gb, session);
				authZ.authorize(Permissions.WRITE_P, gb, false, session);
			} catch (GroupNotKnownException e)
			{
				throw new InternalException("Base group in application form is unknown.");
			}

			if (action.equals(ApplicationActions.ACCEPT_UPDATED))
			{
				if (properApplication == null || properApplication.getId() != id)
					throw new VOApplicationException("Updated application is invalid " +
					"(is missing or has a wrong id");
			} else
			{
				if (properApplication != null)
					throw new VOApplicationException("If action isn't accept with a update" +
					" no updated application is permitted.");
			}

			if (action.equals(ApplicationActions.REMOVE))
			{
				myDb.removeApplication(id, session);
				return;
			} 
			if (!app.getStatus().equals(ApplicationStatus.PENDING))
				throw new VOApplicationException(
						"Application with id "+ app.getId() + " was already processed. " +
				"It is only possible to REMOVE an already processed application");

			if (action.equals(ApplicationActions.ACCEPT) || 
					action.equals(ApplicationActions.ACCEPT_UPDATED))
				acceptApplication(app.getIdentity(), app.getEquivalentIdentity(), 
						base);
			myDb.updateApplication(id, notes, convertStatus(action), session);

			session.commit();

			if (properApplication != null)
				app = properApplication;
			app.setAdminNotes(notes);
			app.setStatus(convertStatus(action));

			mail.sendProcessedApplicationNotification(app, form);
		} finally
		{
			db.releaseSqlSession(session);
		}
	}

	public List<VOApplication> getApplications(Integer formId, String status) 
		throws SecurityException, InternalException, VOApplicationException
	{
		SqlSession session = db.getSqlSession();
		try
		{
			if (formId != null && formId.intValue() < 0)
				formId = null;
			if (formId == null)
				authZ.authorize(Permissions.FULL_READ_P, false, session);
			else
			{
				GroupBean gb = myDb.getAppFormGroup(formId, session);
				try
				{
					authZ.authorize(Permissions.FULL_READ_P, gb, false, session);
				} catch (GroupNotKnownException e)
				{
					throw new VOApplicationException("BUG!! " +
							"Base group of application form " +
					"referenced in query is unknown.");
				}
			}
			return myDb.getApplications(formId, status, session);
		} finally
		{
			db.releaseSqlSession(session);
		}
	}
	
	private void acceptApplication(Identity toAdd, Identity equivalent, Group base) 
		throws VOApplicationException
	{
		SqlSession session = db.getSqlSession();
		
		try
		{

			if (equivalent == null)
				entities.addIdentity(toAdd, true, session);
			else
				entities.addIdentity(toAdd, equivalent, true, session);
			if (equivalent == null && base != null && 
					!base.getName().equals("/"))
				groups.addToGroup(toAdd, base, session);
			session.commit();
		} catch (UVOSException e)
		{
			throw new VOApplicationException("Can't accept application." +
					" Error occured: " + e.toString());
		} finally
		{
			db.releaseSqlSession(session);
		}
		
	}
	
	private ApplicationStatus convertStatus(ApplicationActions src)
	{
		if (src.equals(ApplicationActions.ACCEPT))
			return ApplicationStatus.ACCEPTED;
		if (src.equals(ApplicationActions.ACCEPT_UPDATED))
			return ApplicationStatus.ACCEPTED_UPDATED;
		return ApplicationStatus.REJECTED;
	}

	public void csrProcessedNotification(String csr, boolean accepted, 
			String certificate, boolean sendNotification) 
		throws SecurityException, InternalException, VOApplicationException
	{
		PKCS10CertificationRequest toUpdate;
		try
		{
			toUpdate = X509IdentityHelper.parseCSR(csr);
		} catch (Exception e1)
		{
			throw new VOApplicationException("CSR send in request is " +
					"invalid: " + e1);
		}
		
		SqlSession session = db.getSqlSession();
		
		try
		{

			List<?> apps = myDb.getCSRApplications(session);
			VOApplicationBean found = null;
			for (Object app: apps)
			{
				VOApplicationBean b = (VOApplicationBean) app;
				String t = b.getIdentityValue();
				try
				{
					PKCS10CertificationRequest test = X509IdentityHelper.parseCSR(t);
					if (test.equals(toUpdate))
					{
						found = b;
						break;
					}
				} catch (Exception e)
				{
					log.warn("Found unparseable CSR among applications (id " 
							+ b.getId() + "): " + e);
				}
			}
			if (found == null)
				throw new VOApplicationException("Requested CSR is not " +
				"present in the database");

			VOApplication app = myDb.resolveApplication(found.getId(), session);
			GroupBean gb = myDb.getAppFormGroup(app.getFormId(), session);
			try
			{
				authZ.authorize(Permissions.WRITE_P, gb, false, session);
			} catch (GroupNotKnownException e)
			{
				throw new InternalException("Base group in application " +
				"form is unknown.");
			}

			if (!app.getStatus().equals(ApplicationStatus.PENDING))
				throw new VOApplicationException("Application was " +
				"already processed");

			if (accepted)
			{
				if (certificate == null)
					throw new VOApplicationException("No certificate " +
					"was received.");
				X509IdentityHelper x509H;
				try
				{
					x509H = new X509IdentityHelper(
							new Identity(IdentityType.X509CERT, 
									certificate));
				} catch (InvalidValueException e)
				{
					throw new VOApplicationException(
							"The certificate received " +
							"can't be parsed: " + e);
				}
				String value = certificate;
				if (app.getIdentity().getType().equals(IdentityType.DN))
				{
					value = x509H.getCertificate().getSubjectX500Principal().
					getName(X500Principal.CANONICAL);
				} else if (!app.getIdentity().getType().equals(
						IdentityType.X509CERT))
					throw new VOApplicationException("BUG - CSR was accepted " +
							"as identity of identity type different then" +
					"X509 certificate or DN.");
				myDb.updateCSRApplication(found.getId(), 
						ApplicationStatus.valueOf(found.getStatus()), 
						false, value, session);
			} else {
				myDb.updateCSRApplication(found.getId(), 
						ApplicationStatus.CSR_REJECTED, true, 
						found.getIdentityValue(), session);
				if (sendNotification)
				{
					VOApplicationForm form = myDb.resolveAppForm(app.getFormId(), session);
					app.setStatus(ApplicationStatus.CSR_REJECTED);
					app.setAdminNotes("Certificate signing request which was " +
							"linked with this application was rejected by the CA. " +
					"Therefore the application was rejected too.");
					mail.sendProcessedApplicationNotification(app, form);
				}
			}
			
			session.commit();
		} finally
		{
			db.releaseSqlSession(session);
		}
	}
}

==> unicore-uvos-server-1.6.0-0.src.tar.gz: src/main/java/pl/edu/icm/unicore/uvos/engine/VOApplicationModule.java <==
/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Mar 18, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.engine;

import java.util.List;

import org.apache.ibatis.session.SqlSession;
import org.apache.log4j.Logger;
import org.bouncycastle.jce.PKCS10CertificationRequest;

import eu.emi.security.authn.x509.impl.X500NameUtils;
import eu.unicore.util.Log;

import pl.edu.icm.unicore.uvos.api.ApplicationActions;
import pl.edu.icm.unicore.uvos.api.ApplicationStatus;
import pl.edu.icm.unicore.uvos.api.Group;
import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.api.IdentityType;
import pl.edu.icm.unicore.uvos.api.Permissions;
import pl.edu.icm.unicore.uvos.api.VOApplication;
import pl.edu.icm.unicore.uvos.api.VOApplicationForm;
import pl.edu.icm.unicore.uvos.api.exceptions.GroupNotKnownException;
import pl.edu.icm.unicore.uvos.api.exceptions.IdentityNotKnownException;
import pl.edu.icm.unicore.uvos.api.exceptions.InternalException;
import pl.edu.icm.unicore.uvos.api.exceptions.InvalidValueException;
import pl.edu.icm.unicore.uvos.api.exceptions.SecurityException;
import pl.edu.icm.unicore.uvos.api.exceptions.UVOSException;
import pl.edu.icm.unicore.uvos.api.exceptions.UnsupportedTypeException;
import pl.edu.icm.unicore.uvos.api.exceptions.VOApplicationException;
import pl.edu.icm.unicore.uvos.db.DB;
import pl.edu.icm.unicore.uvos.db.DBApps;
import pl.edu.icm.unicore.uvos.db.DBEntities;
import pl.edu.icm.unicore.uvos.db.DBGroupResolver;
import pl.edu.icm.unicore.uvos.db.DBGroups;
import pl.edu.icm.unicore.uvos.engine.api.VOApplicationInterface;
import pl.edu.icm.unicore.uvos.idhelpers.IdentityHelperFactory;
import pl.edu.icm.unicore.uvos.idhelpers.IdentityTypeHelper;
import pl.edu.icm.unicore.uvos.idhelpers.X509IdentityHelper;
import pl.edu.icm.unicore.uvos.mail.MailSender;
import pl.edu.icm.unicore.uvos.model.GroupBean;
import pl.edu.icm.unicore.uvos.model.VOApplicationBean;
import pl.edu.icm.unicore.uvos.util.UVOSServerProperties;

/**
 * Class which handles processing of VO applications (or join requests). 
 * @author K. Benedyczak
 */
public class VOApplicationModule implements VOApplicationInterface
{
	private DB db;
	private DBGroupResolver groupResolver;
	private DBEntities entities;
	private DBApps myDb;
	private InternalAuthorization authZ;
	private MailSender mail;
	private DBGroups groups;
	private static final Logger log = Log.getLogger(UVOSServerProperties.LOG_PFX, VOApplicationModule.class);
	
	public VOApplicationModule(DB db, DBGroupResolver groupResolver, DBEntities entities, DBApps myDb,
			InternalAuthorization authZ, MailSender mail, DBGroups groups)
	{
		this.db = db;
		this.myDb = myDb;
		this.authZ = authZ;
		this.mail = mail;
		this.groups = groups;
		this.groupResolver = groupResolver;
		this.entities = entities;
	}

	//
	// public API implementation ******************************************************
	//
	public void updateApplicationForm(VOApplicationForm applicationDef, boolean update) throws SecurityException, InternalException, 
			VOApplicationException
	{
		int baseGroup;
		SqlSession session = db.getSqlSession();
		try
		{
			try
			{
				GroupBean gb;
				if (!update)
					gb = groupResolver.resolveGroup(applicationDef.getBaseGroup(), null, session); 
				else
				{
					gb = groupResolver.resolveGroup(applicationDef.getBaseGroup(), null, session);
					baseGroup = gb.getId();
					gb = myDb.getAppFormGroup(applicationDef.getId(), session);
					if (baseGroup != gb.getId())
						throw new VOApplicationException("Application form update " +
						"can not change the group of the original application.");
				}
				authZ.authorize(Permissions.WRITE_P, gb, false, session);
				baseGroup = gb.getId();
			} catch (GroupNotKnownException e)
			{
				throw new VOApplicationException("Requested base group is unknown.");
			}

			if (update)
				myDb.updateAppForm(applicationDef, baseGroup, session);
			else
				myDb.addAppForm(applicationDef, session);
			session.commit();
		} finally
		{
			db.releaseSqlSession(session);
		}
	}

	public void removeApplicationForm(int id) 
		throws SecurityException, InternalException, VOApplicationException
	{
		SqlSession session = db.getSqlSession();
		try
		{
			try
			{
				GroupBean gb = myDb.getAppFormGroup(id, session);
				authZ.authorize(Permissions.WRITE_P, gb, false, session);
			} catch (GroupNotKnownException e)
			{
				throw new InternalException("Base group in application form is unknown.");
			}
			myDb.removeAppForm(id, session);
			session.commit();
		} finally
		{
			db.releaseSqlSession(session);
		}
	}

	public List<VOApplicationForm> getApplicationForms() 
		throws SecurityException, InternalException
	{
		SqlSession session = db.getSqlSession();
		try
		{
			return myDb.listAppForms(session);
		} finally
		{
			db.releaseSqlSession(session);
		}
	}

	public void submitApplication(VOApplication application) 
		throws SecurityException, InternalException, VOApplicationException
	{
		SqlSession session = db.getSqlSession();
		try
		{
			//this also checks if app form exists and is valid
			VOApplicationForm form = myDb.resolveAppForm(application.getFormId(), session);
			
			Identity id = application.getIdentity();
			IdentityTypeHelper helper = null;
			try
			{
				if (application.isCSR())
					X509IdentityHelper.parseCSR(id.getValue());
				else
					helper = IdentityHelperFactory.getHelper(id);
			} catch (Exception e)
			{
				throw new VOApplicationException("The identity value in " +
					"application is invalid for the specified type:" + 
					e);
			}
			Identity equivalent = application.getEquivalentIdentity();
			if (equivalent != null)
			{
				try
				{
					if (!authZ.isSelf(equivalent, session))
						throw new SecurityException("VO Applications for entry equivalent " +
							"to the existing one are available only for users " +
							"authenticated as this existing one.");
				} catch (IdentityNotKnownException e)
				{
					throw new VOApplicationException("Requested equivalent identity is not known.");
				} catch (InvalidValueException e)
				{
					throw new VOApplicationException("Requested equivalent identity is invalid.");
				}
			}
			
			myDb.addApplication(application, helper, session);
			session.commit();
			
			mail.sendNewApplicationNotification(application, form);
		} catch (UnsupportedTypeException e)
		{
			throw new VOApplicationException("Identity type requested in " +
					"application is not known");
		} finally
		{
			db.releaseSqlSession(session);
		}
	}

	public void processApplication(int id, ApplicationActions action, String notes, 
			boolean sendConfirmation, 
			VOApplication properApplication) 
		throws SecurityException, InternalException, VOApplicationException
	{
		SqlSession session = db.getSqlSession();

		try
		{
			VOApplication app = myDb.resolveApplication(id, session);
			GroupBean gb = myDb.getAppFormGroup(app.getFormId(), session);
			VOApplicationForm form = myDb.resolveAppForm(app.getFormId(), session);
			Group base;
			try
			{
				base = groupResolver.resolveGroupBean(gb, session);
				authZ.authorize(Permissions.WRITE_P, gb, false, session);
			} catch (GroupNotKnownException e)
			{
				throw new InternalException("Base group in application form is unknown.");
			}

			if (action.equals(ApplicationActions.ACCEPT_UPDATED))
			{
				if (properApplication == null || properApplication.getId() != id)
					throw new VOApplicationException("Updated application is invalid " +
					"(is missing or has a wrong id");
			} else
			{
				if (properApplication != null)
					throw new VOApplicationException("If action isn't accept with a update" +
					" no updated application is permitted.");
			}

			if (action.equals(ApplicationActions.REMOVE))
			{
				myDb.removeApplication(id, session);
				return;
			} 
			if (!app.getStatus().equals(ApplicationStatus.PENDING))
				throw new VOApplicationException(
						"Application with id "+ app.getId() + " was already processed. " +
				"It is only possible to REMOVE an already processed application");

			if (action.equals(ApplicationActions.ACCEPT) || 
					action.equals(ApplicationActions.ACCEPT_UPDATED))
				acceptApplication(app.getIdentity(), app.getEquivalentIdentity(), 
						base);
			myDb.updateApplication(id, notes, convertStatus(action), session);

			session.commit();

			if (properApplication != null)
				app = properApplication;
			app.setAdminNotes(notes);
			app.setStatus(convertStatus(action));

			mail.sendProcessedApplicationNotification(app, form);
		} finally
		{
			db.releaseSqlSession(session);
		}
	}

	public List<VOApplication> getApplications(Integer formId, String status) 
		throws SecurityException, InternalException, VOApplicationException
	{
		SqlSession session = db.getSqlSession();
		try
		{
			if (formId != null && formId.intValue() < 0)
				formId = null;
			if (formId == null)
				authZ.authorize(Permissions.FULL_READ_P, false, session);
			else
			{
				GroupBean gb = myDb.getAppFormGroup(formId, session);
				try
				{
					authZ.authorize(Permissions.FULL_READ_P, gb, false, session);
				} catch (GroupNotKnownException e)
				{
					throw new VOApplicationException("BUG!! " +
							"Base group of application form " +
					"referenced in query is unknown.");
				}
			}
			return myDb.getApplications(formId, status, session);
		} finally
		{
			db.releaseSqlSession(session);
		}
	}
	
	private void acceptApplication(Identity toAdd, Identity equivalent, Group base) 
		throws VOApplicationException
	{
		SqlSession session = db.getSqlSession();
		
		try
		{

			if (equivalent == null)
				entities.addIdentity(toAdd, true, session);
			else
				entities.addIdentity(toAdd, equivalent, true, session);
			if (equivalent == null && base != null && 
					!base.getName().equals("/"))
				groups.addToGroup(toAdd, base, session);
			session.commit();
		} catch (UVOSException e)
		{
			throw new VOApplicationException("Can't accept application." +
					" Error occured: " + e.toString());
		} finally
		{
			db.releaseSqlSession(session);
		}
		
	}
	
	private ApplicationStatus convertStatus(ApplicationActions src)
	{
		if (src.equals(ApplicationActions.ACCEPT))
			return ApplicationStatus.ACCEPTED;
		if (src.equals(ApplicationActions.ACCEPT_UPDATED))
			return ApplicationStatus.ACCEPTED_UPDATED;
		return ApplicationStatus.REJECTED;
	}

	public void csrProcessedNotification(String csr, boolean accepted, 
			String certificate, boolean sendNotification) 
		throws SecurityException, InternalException, VOApplicationException
	{
		PKCS10CertificationRequest toUpdate;
		try
		{
			toUpdate = X509IdentityHelper.parseCSR(csr);
		} catch (Exception e1)
		{
			throw new VOApplicationException("CSR send in request is " +
					"invalid: " + e1);
		}
		
		SqlSession session = db.getSqlSession();
		
		try
		{

			List<?> apps = myDb.getCSRApplications(session);
			VOApplicationBean found = null;
			for (Object app: apps)
			{
				VOApplicationBean b = (VOApplicationBean) app;
				String t = b.getIdentityValue();
				try
				{
					PKCS10CertificationRequest test = X509IdentityHelper.parseCSR(t);
					if (test.equals(toUpdate))
					{
						found = b;
						break;
					}
				} catch (Exception e)
				{
					log.warn("Found unparseable CSR among applications (id " 
							+ b.getId() + "): " + e);
				}
			}
			if (found == null)
				throw new VOApplicationException("Requested CSR is not " +
				"present in the database");

			VOApplication app = myDb.resolveApplication(found.getId(), session);
			GroupBean gb = myDb.getAppFormGroup(app.getFormId(), session);
			try
			{
				authZ.authorize(Permissions.WRITE_P, gb, false, session);
			} catch (GroupNotKnownException e)
			{
				throw new InternalException("Base group in application " +
				"form is unknown.");
			}

			if (!app.getStatus().equals(ApplicationStatus.PENDING))
				throw new VOApplicationException("Application was " +
				"already processed");

			if (accepted)
			{
				if (certificate == null)
					throw new VOApplicationException("No certificate " +
					"was received.");
				X509IdentityHelper x509H;
				try
				{
					x509H = new X509IdentityHelper(
							new Identity(IdentityType.X509CERT, 
									certificate));
				} catch (InvalidValueException e)
				{
					throw new VOApplicationException(
							"The certificate received " +
							"can't be parsed: " + e);
				}
				String value = certificate;
				if (app.getIdentity().getType().equals(IdentityType.DN))
				{
					value = X500NameUtils.getComparableForm(
							x509H.getCertificate().getSubjectX500Principal().getName());
				} else if (!app.getIdentity().getType().equals(
						IdentityType.X509CERT))
					throw new VOApplicationException("BUG - CSR was accepted " +
							"as identity of identity type different then" +
					"X509 certificate or DN.");
				myDb.updateCSRApplication(found.getId(), 
						ApplicationStatus.valueOf(found.getStatus()), 
						false, value, session);
			} else {
				myDb.updateCSRApplication(found.getId(), 
						ApplicationStatus.CSR_REJECTED, true, 
						found.getIdentityValue(), session);
				if (sendNotification)
				{
					VOApplicationForm form = myDb.resolveAppForm(app.getFormId(), session);
					app.setStatus(ApplicationStatus.CSR_REJECTED);
					app.setAdminNotes("Certificate signing request which was " +
							"linked with this application was rejected by the CA. " +
					"Therefore the application was rejected too.");
					mail.sendProcessedApplicationNotification(app, form);
				}
			}
			
			session.commit();
		} finally
		{
			db.releaseSqlSession(session);
		}
	}
}
